"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
from constants import *
from helpers import convertor


def get_namespace_info(cluster_obj, namespace_list, node_addresses):

    if node_addresses is None:
        node_addresses = cluster_obj.get_node_list()
    else:
        node_addresses = node_addresses.split(',')

    namespace_info = dict()
    namespace_list = namespace_list.replace(' ', '')
    namespace_list = convertor.text_to_list(namespace_list, ',')

    for namespace_name in namespace_list:
        objects = 0
        repl_factor = None
        expired_objects = 0
        evicted_objects = 0
        used_bytes_disk = 0
        total_bytes_disk = 0
        used_bytes_memory = 0
        total_bytes_memory = 0
        aerospike_greater_than_3 = False
        least_available_pct = dict(value=None, node=None)

        if cluster_obj.get_latest_build_no().startswith('3'):
            aerospike_greater_than_3 = True
            master_objects = 0
            prole_objects = 0

        for node_address in node_addresses:
            ns_obj = cluster_obj.get_namespace(namespace_name, node_address)
            if ns_obj is not None:
                ns_obj.update_statistics()
                subset_of_stats = ["expired-objects", "evicted-objects",
                "repl-factor", "total-bytes-disk", "used-bytes-disk",
                "total-bytes-memory", "used-bytes-memory", "available_pct"]
                if aerospike_greater_than_3 is True:
                    subset_of_stats.extend(["master-objects", "prole-objects"])
                else:
                    subset_of_stats.append("objects")
                stats = ns_obj.get_subset_of_statistics(subset_of_stats)
                if aerospike_greater_than_3 is True:
                    master_objects += stats.get_as_int("master-objects", 0)
                    prole_objects += stats.get_as_int("prole-objects", 0)
                else:
                    objects += stats.get_as_int("objects", 0)
                repl_factor = stats.get_as_int("repl-factor")
                expired_objects += stats.get_as_int("expired-objects", 0)
                evicted_objects += stats.get_as_int("evicted-objects", 0)
                used_bytes_disk += stats.get_as_int("used-bytes-disk", 0)
                total_bytes_disk += stats.get_as_int("total-bytes-disk", 0)
                used_bytes_memory += stats.get_as_int("used-bytes-memory", 0)
                total_bytes_memory += stats.get_as_int("total-bytes-memory", 0)
                available_pct = stats.get_as_int("available_pct")
                if available_pct is not None and (least_available_pct['value'] is None or available_pct < least_available_pct['value']):
                    least_available_pct['value'] = available_pct
                    least_available_pct['node'] = node_address

        namespace_info[namespace_name] = {
            "repl-factor": repl_factor,
            "expired-objects": expired_objects,
            "evicted-objects": evicted_objects,
            "disk": {
                "total-bytes-disk": total_bytes_disk,
                "used-bytes-disk": used_bytes_disk
            },
            "memory": {
                "total-bytes-memory": total_bytes_memory,
                "used-bytes-memory": used_bytes_memory
            },
            "least_available_pct": least_available_pct,
            "cluster_status": cluster_obj.get_status()
        }
        if aerospike_greater_than_3 is True:
            namespace_info[namespace_name].update(
                                {"master-objects": master_objects,
                                  "prole-objects": prole_objects})
        else:
            namespace_info[namespace_name].update({"objects": objects})

    return namespace_info


def get_namespace_info_per_node(cluster_obj, namespace_name, node_list=None):

    if node_list is None:
        node_list = cluster_obj.get_node_list()
    else:
        node_list = node_list.replace(' ', '')
        node_list = convertor.text_to_list(node_list, ',')

    namespace_info = dict()

    for node_address in node_list:
        namespace_info[node_address] = dict(name=namespace_name)
        ns_obj = cluster_obj.get_namespace(namespace_name, node_address)

        if ns_obj is None:
            namespace_info[node_address]['node_status'] = "off"
            continue

        ns_obj.update_statistics()

        namespace_info[node_address]["memory"] = ns_obj.get_memory_stats()
        namespace_info[node_address]["disk"] = ns_obj.get_disk_stats()

        subset_of_stats = ["expired-objects", "evicted-objects", "repl-factor",
        "memory-size", "free-pct-memory", "max-void-time", "hwm-breached",
        "default-ttl", "max-ttl", "max-ttl", "enable-xdr", "stop-writes",
        "available_pct", "stop-writes-pct", "hwm-breached", "single-bin",
        "data-in-memory", "type", "node_status"]

        aerospike_build = cluster_obj.get_latest_build_no()

        if aerospike_build.startswith('3'):
            subset_of_stats.extend(["master-objects", "prole-objects"])
        else:
            subset_of_stats.append("objects")
        stats = ns_obj.get_subset_of_statistics(subset_of_stats)

        namespace_info[node_address].update(stats)
        #namespace_info["configuration"] = ns_obj.get_configuration()

        namespace_info[node_address]["disk-pct"] = {
            "free-pct-disk": ns_obj.get_statistic("free-pct-disk"),
            "high-water-disk-pct": ns_obj.get_statistic("high-water-disk-pct")
             }

        namespace_info[node_address]["memory-pct"] = {
            "free-pct-memory": ns_obj.get_statistic("free-pct-memory"),
            "high-water-memory-pct": ns_obj.get_statistic("high-water-memory-pct")
             }
    return namespace_info


def get_stats(cluster_obj, namespace_name, node_address, stats_type="all"):

    #namespace_info = dict(name=namespace_name, node=node_address)
    namespace_info = dict()
    ns_obj = cluster_obj.get_namespace(namespace_name, node_address)

    if ns_obj is None:
        namespace_info['node_status'] = "off"
    else:
        stats = ns_obj.get_updated_statistics() if stats_type == "all" else ns_obj.get_modified_statistics()
        namespace_info.update(stats)

    return namespace_info


def get_index_stats(cluster_obj, namespace_name, index_name, node_address, stats_type="all"):

    #namespace_info = dict(namespace=namespace_name, index=index_name, node=node_address)
    namespace_info = dict()
    ns_obj = cluster_obj.get_namespace(namespace_name, node_address)

    if ns_obj is None:
        namespace_info['node_status'] = "off"
    else:
        stats = ns_obj.get_index_statistics(index_name, stats_type)
        stats['namespace'] = namespace_name
        namespace_info.update(stats)

    return namespace_info


def get_storage_info(cluster_obj, namespace_name):

    cluster_status = cluster_obj.get_status()
    if cluster_status == "off":
        return {"cluster_status": "off"}

    node_addresses = cluster_obj.get_node_list()
    storage_types = {}
    storage_devices = {}

    for node_address in node_addresses:
        ns_obj = cluster_obj.get_namespace(namespace_name, node_address)
        if ns_obj is not None:
            storage_type = ns_obj.get_statistic("type")
            if storage_type != "n/s":
                storage = storage_types.get(storage_type, [])
                storage.append(node_address)
                storage_types[storage_type] = storage
            storage_device = ns_obj.get_statistic("dev")
            if storage_device != "n/s":
                device = storage_devices.get(storage_device, [])
                device.append(node_address)
                storage_devices[storage_device] = device

    synced_status = True if len(storage_types) == 1 else False
    storage_info = {"cluster_status": "on"}
    storage_info["synced"] = synced_status
    storage_info["storage"] = storage_types
    storage_info["devices"] = storage_devices
    return storage_info
